package com.dsic.gj_erp.handler;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateRange;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.dsic.gj_erp.bean.SpringContextHolder;
import com.dsic.gj_erp.bean.jcsj.*;
import com.dsic.gj_erp.bean.jhgk.DmSygdMx;
import com.dsic.gj_erp.handler.dto.RealCapacity;
import com.dsic.gj_erp.service.jcsj.EmGcrlService;
import com.dsic.gj_erp.service.jcsj.EmSbcnpService;
import com.dsic.gj_erp.service.jcsj.EmSbjbbService;
import com.dsic.gj_erp.service.jcsj.EmSbjgryService;
import lombok.Getter;
import lombok.Setter;

import java.util.*;
import java.util.stream.Collectors;

@Getter
@Setter
public class ComputePlan {

    private static ComputePlan instance = null;

    List<Object> workerList;

    Map<String, EmSbcnp> baseCapacity;

    Map<String,List<EmSbjgry>> empSizeByDay;

    List<EmGcrl> gcrlList;

    Map<String,Map<String,List<RealCapacity>>>  realCapacityMap;

    List<EmSbjbb> deviceList;
    List<EmSbcnp> sbcnList;


    JSONObject realDeviceList;

    List<DmSygdMx> sygdMxList;

    Map<String,List<DmBom>> dmBomList;

    /**
     * 计算实际产能,默认未来三个月
     */
    public static void computeRealCapacity() {
        Date from = DateUtil.beginOfMonth(DateUtil.offsetMonth(new Date(), 1));
        Date to = DateUtil.beginOfMonth(DateUtil.offsetMonth(new Date(), 4));
        computeRealCapacity(from, to);
    }

    /**
     * 计算制定日期范围内每天实际产能
     *
     * @param from 开始日期
     * @param to   结束日期
     */
    public static void computeRealCapacity(Date from, Date to) {
        Map<String,Map<String,List<RealCapacity>>> _realCapacity = instance().realCapacityMap = new TreeMap<>();

        //加载工厂日历
        initGcrl();
        //加载设备信息
        initDevice();
        //加载工作人员
        initEmp(from, to);
        //加载产能信息
        initSbcn();
        //计算周期内按天分解的实际可用设备产能
        initRealDeviceList(from, to);

        //可用设备
        JSONObject realDeviceList = instance().realDeviceList;

        List<EmGcrl> gcrlList = instance().gcrlList.stream()
                .filter(item -> "1".equals(item.getXxr()))
                .filter(item -> DateUtil.isIn(DateUtil.parse(item.getGl(), "yyyy/MM/dd"), from, to))
                .collect(Collectors.toList());
//        Collections.sort(gcrlList,Comparator.comparing(EmGcrl::getGl,(t1,t2)->t1.compareTo(t2)));
        gcrlList.forEach(gcrl -> {
            if (realDeviceList .size()<1 || ObjectUtil.isEmpty(realDeviceList.getJSONArray(gcrl.getGl()))) {
                return;
            }
            //日可用设备
            for (int i = 0; i < realDeviceList.getJSONArray(gcrl.getGl()).size(); i++) {
                JSONObject deviceJson = realDeviceList.getJSONArray(gcrl.getGl()).getJSONObject(i);
                //日实际产能
                //todo 设备和产能对应关系,找出当前设备产能 (日期  产能 工序 图纸名称)
                RealCapacity realCapacity = initRealCapacity(gcrl, deviceJson);
                if (realCapacity.getCapacity()==0){
                    continue;
                }
               String kwgx= realCapacity.getKw()+realCapacity.getGx();
                if (_realCapacity.containsKey(gcrl.getGl())) {
                    if(_realCapacity.get(gcrl.getGl()).containsKey(kwgx)){
                        _realCapacity.get(gcrl.getGl()).get(kwgx).add(realCapacity);
                    }else {
                        List<RealCapacity> list=new ArrayList<>();
                        list.add(realCapacity);
                        _realCapacity.get(gcrl.getGl()).put(kwgx, list);
                    }
                } else {
//                    JSONObject object = new JSONObject();
//                    object.put(realCapacity.getDeviceName(), realCapacity.getCapacity());
                    Map<String,List<RealCapacity>> kwANDgX= new TreeMap<>();
                    List<RealCapacity> list=new ArrayList<>();
                    list.add(realCapacity);
                    kwANDgX.put(kwgx, list);
                    _realCapacity.put(gcrl.getGl(), kwANDgX);
                }
            }
        });
    }

    /**
     * 加载工厂日历
     */
    public static void initGcrl() {
//        if (instance().gcrlList == null){
            instance().gcrlList = getBean(EmGcrlService.class).getList(new JSONObject());
//        }
    }

    /**
     * 加载设备产能
     */
    public static void initSbcn() {
//        if (instance().sbcnList == null){
            instance().sbcnList = getBean(EmSbcnpService.class).getList(new JSONObject());
//        }
    }


    public static void initDeviceCapacity() {
        if (instance().baseCapacity == null) {
            instance().baseCapacity = getBean(EmSbcnpService.class).getMap();
        }
    }

    public static void initRealDeviceList(Date from, Date to) {
        initRealDeviceList(from,to,false);
    }

    /**
     * 按日期分解可用设备
     * @param from 开始日期
     * @param to 结束日期
     * @param flag 强制重置
     */
    public static void initRealDeviceList(Date from, Date to,boolean flag) {

//        if (ObjectUtil.isEmpty(instance().realDeviceList)||flag) {
            JSONObject realDeviceList = instance().realDeviceList = new JSONObject();
            List<EmSbjbb> list = instance().deviceList;

            List<EmSbjbb> canUseDevice = list.stream().filter(item -> "0".equals(item.getSbyxzt())).collect(Collectors.toList());
            //计算可用设备的可用日期
            Map<String, List<EmSbjgry>> _empSizeByDay = instance().empSizeByDay;

            canUseDevice.forEach(item -> {
                //未定义图纸及工序的设备，无法参加排产
                if (StrUtil.hasEmpty(item.getGx())){
                    return;
                }
                //维修时间
                List<DateTime> dateTimes ;
                if (StrUtil.isNotEmpty(item.getWhsj())){
                    String[] whArr = item.getWhsj().split("_");
                    if (whArr.length < 2) {
                        return;
                    }
                    DateRange dateRange = new DateRange(from, to, DateField.DAY_OF_YEAR);//需要放到循环内，放外边不能重复使用
                    DateRange tmpRange = new DateRange(DateUtil.parse(whArr[0], "yyyy/MM/dd"), DateUtil.parse(whArr[1], "yyyy/MM/dd"), DateField.DAY_OF_YEAR);
                    dateTimes = DateUtil.rangeNotContains( tmpRange,dateRange);
                }else {
                    dateTimes=DateUtil.rangeToList(from, to, DateField.DAY_OF_YEAR);
                }

                dateTimes.forEach(date -> {
                    String dateStr = date.toString("yyyy/MM/dd");
                    //todo 根据人员请假情况,计算当前日期设备是否可用
//                    if (ObjectUtil.isEmpty(_empSizeByDay)||_empSizeByDay.get(dateStr).size()<=0){
//                        return;
//                    }
                    if (realDeviceList.get(dateStr) == null) {
                        JSONArray arr = new JSONArray();
                        arr.add(item);
                        realDeviceList.put(dateStr, arr);
                    } else {
                        realDeviceList.getJSONArray(dateStr).add(item);
                    }
                });
            });
//        }
    }

    public static void initEmp(Date from,Date to){
        //设备编号? 人员 休假日期 状态（在职）
        if (ObjectUtil.isEmpty(instance().empSizeByDay)){
           Map<String,List<EmSbjgry>> empSizeByDay= instance().empSizeByDay=new HashMap<>();
            //1.全部工作人员 状态（在职）
            List<EmSbjgry> empList  = getBean(EmSbjgryService.class).list(Wrappers.<EmSbjgry>lambdaQuery().eq(EmSbjgry::getZt,"01"));
            //2.根据休息时间范围取差集

            empList.forEach(emp->{
                DateRange dateRange = new DateRange(from, to, DateField.DAY_OF_YEAR);//需要放到循环内，放外边不能重复使用
                DateRange tmpRange = new DateRange(DateUtil.parse(emp.getXjsjKs(), "yyyy/MM/dd"), DateUtil.parse(emp.getXjsjJs(), "yyyy/MM/dd"), DateField.DAY_OF_YEAR);
                List<DateTime> dateTimes = DateUtil.rangeNotContains( tmpRange,dateRange);
                dateTimes.forEach(date->{
                    String dateStr = date.toString("yyyy/MM/dd");
                    if (ObjectUtil.isEmpty(empSizeByDay)||empSizeByDay.get(dateStr) == null) {
                        List<EmSbjgry> arr = new ArrayList<>();
                        arr.add(emp);
                        empSizeByDay.put(dateStr, arr);
                    } else {
                        empSizeByDay.get(dateStr).add(emp);
                    }
                });
            });
        }


    }

    /**
     * 加载设备信息表
     */
    public static void initDevice() {
        instance().deviceList = getBean(EmSbjbbService.class).list(Wrappers.<EmSbjbb>lambdaQuery().like(EmSbjbb::getScbs,"1"));
    }

    public synchronized static ComputePlan instance() {
        if (instance == null) {
            instance = new ComputePlan();
        }
        return instance;
    }

    private static RealCapacity initRealCapacity(EmGcrl gcrl, JSONObject deviceJson) {
        //设备产能，根据图号和工序去重，注意：同图号和工序下，不同组立对应的产能要一致
//        List<EmSbcnp> list = instance().sbcnList.stream()
//                .collect(Collectors.collectingAndThen(Collectors.toCollection(()->
//                        new TreeSet<>(Comparator.comparing(o->o.getSbbh()+o.getGx()))),ArrayList::new));
//        String gx=deviceJson.getString("gx");
//        String tzbh=deviceJson.getString("tzbh");
        RealCapacity capacity = new RealCapacity();
//        list.forEach(sbcnp->{
//            if (sbcnp.getGx().equals(gx)&&
//                    sbcnp.getTzbh().equals(tzbh)){
        Double degs=deviceJson.getDouble("degs");
        Double degsYb=deviceJson.getDouble("degsYb");
        degs=ObjectUtil.isEmpty(degs)?0.0:degs;
        degsYb=ObjectUtil.isEmpty(degsYb)?0.0:degsYb;
        if (StrUtil.isEmpty(gcrl.getPb())){
            capacity.setCapacity(degs * 7);
        }
                if ("3".equals(gcrl.getPb())) {
                    capacity.setCapacity(degs * gcrl.getBbgs() + degsYb * gcrl.getYbgs());
                }
                if ("1".equals(gcrl.getPb())) {
                    capacity.setCapacity(degs * gcrl.getBbgs());
                }
                if ("2".equals(gcrl.getPb())) {
                    capacity.setCapacity(degsYb * gcrl.getYbgs());
                }
//            }
//        });
        capacity.setDeviceName(deviceJson.getString("sbbh"));
        capacity.setDate(gcrl.getGl());
        capacity.setGx(deviceJson.getString("gx"));
        capacity.setKw(deviceJson.getString("kw"));
        return capacity;
    }

    private static <T> T getBean(Class<T> clazz) {
        return SpringContextHolder.getBean(clazz);
    }


}
