package com.dsic.gj_erp.service.txzz;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dsic.gj_erp.bean.ResultBean;
import com.dsic.gj_erp.bean.sy.SyDictItem;
import com.dsic.gj_erp.bean.txzz.DmMateria;
import com.dsic.gj_erp.mapper.txzz.scby.DmMateriaMapper;
import com.dsic.gj_erp.service.sy.SyDictItemService;
import com.dsic.gj_erp.util.ConditionSelect;
import lombok.AllArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * 物品基本表 服务实现类
 * </p>
 *
 * @author xhj
 * @since 2021-03-29
 */
@Service
@AllArgsConstructor
public class DmMateriaServiceImpl extends ServiceImpl<DmMateriaMapper, DmMateria> implements DmMateriaService {

    private final SyDictItemService syDictItemService;

    public Map getBase(Map<String, Object> params) {
        Map dictMap = new HashMap();
        //管系类型
        List<Map<String, Object>> gxlx = syDictItemService.getCodeByString("GXLX");
        dictMap.put("gxlx", gxlx);
        return dictMap;
    }

    @Override
    public Map getjcsjWpBase(Map<String, Object> params) {
        // 1 物品级别 wpjb  2 物品分类 WPLX  3购置类型（目前没有这个字段） 4 计量单位 jldw            5 工艺类别（）  6用户分类码  7 仓库 （查表）
        Map dictMap =new HashMap();
        List<SyDictItem> wpjb = syDictItemService.list(new QueryWrapper<SyDictItem>().eq("dict_code", "WPJB"));
        List<SyDictItem> wplx = syDictItemService.list(new QueryWrapper<SyDictItem>().eq("dict_code", "WPLX").in("item_code","0201","0202","0203"));
        List<SyDictItem> jldw = syDictItemService.list(new QueryWrapper<SyDictItem>().eq("dict_code", "JLDW"));
        List<SyDictItem> fjsyfs = syDictItemService.list(new QueryWrapper<SyDictItem>().eq("dict_code", "FJSYFS"));
//        List<DmDepart> dept=dmDepartService.list(new QueryWrapper<DmDepart>().in("institution_code","03","04","05").eq("system_code","IM"));
        dictMap.put("wpjb",wpjb);
        dictMap.put("wplx",wplx);
        dictMap.put("jldw",jldw);
        dictMap.put("fjsyfs",fjsyfs);
        return dictMap;
    }

    /**
     * 物品条件全部查询
     * @author 张恩铭
     * @param dmMateria
     * @return
     */
    @Override
    public ResultBean<List<DmMateria>> getAllCon(DmMateria dmMateria) {
        ConditionSelect<DmMateria> conditionSelect = new ConditionSelect<>(dmMateria, new QueryWrapper<DmMateria>(), baseMapper);
        ResultBean resultBean = new ResultBean<List<DmMateria>>();
        resultBean.setData(conditionSelect.getResult());
        return resultBean;
    }

    /**
     * 物品条件管系查询
     * @author 张恩铭
     * @param dmMateria
     * @return
     */
    @Override
    public ResultBean<List<DmMateria>> getCon(DmMateria dmMateria) {
        QueryWrapper<DmMateria> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("gj_flag", 1);
        if(StringUtils.isNotBlank(dmMateria.getMatNo())){
            queryWrapper.like("mat_no",dmMateria.getMatNo());
        }
        if(StringUtils.isNotBlank(dmMateria.getMatName())){
            queryWrapper.like("mat_name",dmMateria.getMatName());
        }
        if(StringUtils.isNotBlank(dmMateria.getMatGrd())){
            queryWrapper.like("mat_grd",dmMateria.getMatGrd());
        }
        if(StringUtils.isNotBlank(dmMateria.getMatSpec())){
            queryWrapper.like("mat_spec",dmMateria.getMatSpec());
        }
        if(StringUtils.isNotBlank(dmMateria.getMatVersion())){
            queryWrapper.like("mat_version",dmMateria.getMatVersion());
        }
        if(StringUtils.isNotBlank(dmMateria.getInstitutionCode())){
            queryWrapper.like("institution_code",dmMateria.getInstitutionCode());
        }
        queryWrapper.orderByAsc("mat_no");
        List<DmMateria> result = this.list(queryWrapper);
//        Map dictMap =new HashMap();
//
//        List <String> dictCodes= Arrays.asList("WPJB","WPLX","JLDW");
//        QueryWrapper<SyDictItem> qw1 = new QueryWrapper<SyDictItem>();
//        qw1.in("dict_code", dictCodes);
//        List<SyDictItem> list = syDictItemService.list(qw1);
//        List<SyDictItem> wpjb = list.stream().filter(fsts -> "WPJB".equalsIgnoreCase(fsts.getDictCode())).collect(Collectors.toList());
//        List<SyDictItem> wplx = list.stream().filter(fsts -> "WPLX".equalsIgnoreCase(fsts.getDictCode())).collect(Collectors.toList());
//        List<SyDictItem> jldw = list.stream().filter(fsts -> "JLDW".equalsIgnoreCase(fsts.getDictCode())).collect(Collectors.toList());
//        Map <String,String>wplxMap = wplx.stream().collect(
//                Collectors.toMap(s->s.getItemCode(), s -> s.getItemValue()));
//        for( DmMateria d: result){
//            String a =wplxMap.get(d.getMatCateg());
//            d.setMatCateg(wplxMap.get(d.getMatCateg()));
//            d.setMatLevel("管系附件");
//        }
        ResultBean resultBean = new ResultBean();
        resultBean.setData(result);
        return resultBean;
    }

    /**
     * 物品添加
     * @author 张恩铭
     * @param dmMaterias
     * @return
     */
    @Override
    public ResultBean<Integer> addMateria(DmMateria[] dmMaterias,HttpServletRequest req) {
        int i = 0;
        Date now = new Date();
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy/MM/dd");
        for (DmMateria d : dmMaterias) {
            if(d.getMatNo() == null){
                continue;
            }else {
                List<DmMateria> conditionSelect = this.list(new QueryWrapper<DmMateria>().eq("mat_no", d.getMatNo()));
                if(conditionSelect.size()>0){
                    continue;
                }else{
                    //d.setBzrq(simpleDateFormat.format(now));
                   // d.setGjFlag(1);
                    if(baseMapper.insert(toMateria(d,req,"N")) == 1) i++;
                }
            }
        }
        ResultBean resultBean = new ResultBean<Integer>();
        resultBean.setData(i);
        return resultBean;
    }

    /**
     * 物品修改
     * @author
     * @param dmMaterias
     * @return
     */
    @Override
    public ResultBean<Integer> updateMateria(DmMateria[] dmMaterias, HttpServletRequest req) {
        int i = 0;
        Date now = new Date();
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy/MM/dd");
        for (DmMateria d : dmMaterias) {
            if(baseMapper.updateById(toMateria(d,req,d.getUseStatus())) == 1) i++;


        }
        ResultBean resultBean = new ResultBean<Integer>();
        resultBean.setData(i);
        return resultBean;
    }

    /**
     * 物品删除
     * @author 张恩铭
     * @param dmMaterias
     * @return
     */
    @Override
    public ResultBean<Integer> delMateria(DmMateria[] dmMaterias) {
        int i = 0;
        for (DmMateria d : dmMaterias) {
            if(d.getUseStatus().equals("Y")){
             //   ConditionSelect<DmMateria> conditionSelect = new ConditionSelect<>(d, new QueryWrapper<>(), this.dmMateriaMapper);
              //  if(conditionSelect.getResult().get(0).getUseStatus().equals("Y")){
                    continue;
              //  }
            }else{
                QueryWrapper<DmMateria> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("mat_no", d.getMatNo());
                if(baseMapper.delete(queryWrapper) == 1) i++;
            }
        }
        ResultBean resultBean = new ResultBean();
        resultBean.setData(i);
        return resultBean;
    }

    public DmMateria  toMateria(DmMateria fjExcel, HttpServletRequest req,String zt){
        DmMateria dmMateria = new DmMateria();
        dmMateria.setMatNo(fjExcel.getMatNo());
        dmMateria.setMatName(fjExcel.getMatName());
        dmMateria.setMatSpec(fjExcel.getMatSpec());
        dmMateria.setMatVersion(fjExcel.getMatVersion());
        dmMateria.setMatGrd(fjExcel.getMatGrd());
//        dmMateria.setInstitutionCode(fjExcel.getInstitutionCode());
        dmMateria.setJweight(fjExcel.getJweight());
        dmMateria.setMatArea(fjExcel.getMatArea());
        dmMateria.setMatArea1(fjExcel.getMatArea1());
        dmMateria.setHsUnit(StringUtils.isBlank(fjExcel.getHsUnit())?"个":fjExcel.getHsUnit());
        dmMateria.setZzUnit(dmMateria.getHsUnit());
        dmMateria.setCgUnit(dmMateria.getHsUnit());
//        dmMateria.setMatLevel("F");
//        ;

        if(dmMateria.getMatNo().contains(" II")||dmMateria.getMatNo().contains(" Ⅱ")){
            dmMateria.setFjjb("II");
        }else if(dmMateria.getMatNo().contains(" I")||dmMateria.getMatNo().contains(" Ⅰ")){
            dmMateria.setFjjb("I");
        }
        String ckCode;
        if(StringUtils.isNotBlank(fjExcel.getInstitutionCode())){
            if(fjExcel.getInstitutionCode().contains("安装")||fjExcel.equals("03")){
                dmMateria.setInstitutionCode("03");
                dmMateria.setMatCateg("0201");
                dmMateria.setMatLevel("F");
            }else if (fjExcel.getInstitutionCode().contains("支架")||fjExcel.equals("05")){
                dmMateria.setInstitutionCode("05");
                dmMateria.setMatCateg("0202");
                dmMateria.setMatLevel("Z");
            }else{
                dmMateria.setInstitutionCode("04");
                dmMateria.setMatCateg("0203");
                dmMateria.setMatLevel("F");

            }
        }else{
            dmMateria.setInstitutionCode("04");
            dmMateria.setMatCateg("0203");
            dmMateria.setMatLevel("F");
        }
        dmMateria.setCgDept(fjExcel.getCgDept());
        dmMateria.setCgRy(fjExcel.getCgRy());
        dmMateria.setMainDept1(fjExcel.getMainDept());
        dmMateria.setMainDept(fjExcel.getMainDept());
        dmMateria.setOrderPeriod(fjExcel.getOrderPeriod());
        dmMateria.setGjFlag(1);
        dmMateria.setT1(fjExcel.getT1());
        dmMateria.setT2(fjExcel.getT2());
        dmMateria.setCraftRemark(fjExcel.getCraftRemark());
        dmMateria.setDepartCode("02");
        dmMateria.setUseStatus(zt);
        dmMateria.setBzry((String)req.getAttribute("username"));
        dmMateria.setBzrq(DateUtil.format( DateUtil.date(), "yyyy/MM/dd"));
        dmMateria.setFixedWire(fjExcel.getFixedWire());
        dmMateria.setWeldngNum(fjExcel.getWeldngNum()==null?0:fjExcel.getWeldngNum());
        dmMateria.setRemark(fjExcel.getRemark());
        dmMateria.setUseType(StringUtils.isBlank(fjExcel.getUseType())?"0":fjExcel.getUseType());
        dmMateria.setSafekc(fjExcel.getSafekc());
        dmMateria.setMaxkc(fjExcel.getMaxkc());
        dmMateria.setItemBm(fjExcel.getItemBm());

        return dmMateria;

    }

}
